import click

from savvihub.cli._base import VesslGroup, vessl_argument, vessl_option
from savvihub.cli._util import (
    Endpoint,
    format_string,
    generic_prompter,
    print_data,
    print_table,
    prompt_choices,
    truncate_datetime,
)
from savvihub.cli.organization import organization_name_option
from savvihub.util.constant import SSH_CONFIG_PATH
from savvihub.workspace import (
    backup_workspace,
    connect_workspace_ssh,
    list_workspaces,
    read_workspace,
    restore_workspace,
    update_vscode_remote_ssh,
)


def workspace_id_prompter(
    ctx: click.Context, param: click.Parameter, value: int
) -> int:
    workspaces = list_workspaces()
    return prompt_choices("Workspace", [(x.name, x.id) for x in workspaces])


@click.command(name="workspace", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument("id", type=click.INT, required=True, prompter=workspace_id_prompter)
@organization_name_option
def read(id: int):
    workspace = read_workspace(workspace_id=id)
    print_data(
        {
            "ID": workspace.id,
            "Name": workspace.name,
            "Status": workspace.status,
            "Creator": workspace.created_by.username,
            "Created": truncate_datetime(workspace.created_dt),
            "Cluster": workspace.kernel_cluster.name,
            "Root Volume Size": workspace.volume_mounts.root_volume_size,
            "Resource Spec": {
                "Name": workspace.kernel_resource_spec.name,
                "CPU": workspace.kernel_resource_spec.cpu_limit,
                "Memory": workspace.kernel_resource_spec.memory_limit,
                "GPU": f"{workspace.kernel_resource_spec.gpu_limit} ({workspace.kernel_resource_spec.gpu_type})",
            },
            "Ports": [
                {
                    "Name": x.name,
                    "Port": x.port,
                }
                for x in workspace.ports  # TODO: breaks if `nonprimitive_openapi_model_keys` is removed
            ],
            "SSH": format_string(workspace.endpoints.ssh),
        }
    )
    print(
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@click.option("--cluster", type=click.INT, help="Filter workspaces by cluster ID.")
@click.option(
    "--status", type=click.STRING, multiple=True, help="Filter workspaces by status."
)
@click.option("--show-all", is_flag=True, help="If not set, only show my workspaces.")
@organization_name_option
def list(cluster: int, status: str, show_all: bool):
    workspaces = list_workspaces(
        cluster_id=cluster,
        statuses=status,
        mine=(not show_all),
    )
    print_table(
        workspaces,
        ["ID", "Name", "Status", "Creator", "Created", "Resource"],
        lambda x: [
            x.id,
            x.name,
            x.status,
            x.creator,
            truncate_datetime(x.created_dt),
            x.kernel_resource_spec.name,
        ],
    )


@cli.vessl_command()
def backup():
    backup_workspace()


@cli.vessl_command()
def restore():
    restore_workspace()


@cli.vessl_command()
@vessl_argument("id", type=click.INT, required=True, prompter=workspace_id_prompter)
@vessl_option(
    "-p",
    "--key-path",
    type=click.Path(exists=True),
    required=True,
    prompter=generic_prompter("Private key path"),
)
@organization_name_option
def ssh(id: int, key_path: str):
    connect_workspace_ssh(workspace_id=id, private_key_path=key_path)


@cli.vessl_command()
@vessl_option(
    "-p",
    "--key-path",
    type=click.Path(exists=True),
    required=True,
    prompter=generic_prompter("Private key path"),
)
def vscode(key_path: str):
    update_vscode_remote_ssh(private_key_path=key_path)
    print(f"Updated {SSH_CONFIG_PATH}.")



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmSweep(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'algorithm': 'str',
        'created_dt': 'datetime',
        'early_stopping_spec': 'dict(str, object)',
        'edges': 'OrmSweepEdges',
        'env_vars': 'dict(str, object)',
        'final_cost': 'float',
        'id': 'int',
        'immutable_slug': 'str',
        'kernel_cluster_select_policies': 'dict(str, object)',
        'max_experiment_count': 'int',
        'max_failed_experiment_count': 'int',
        'message': 'str',
        'name': 'str',
        'objective': 'dict(str, object)',
        'objective_step_median': 'dict(str, object)',
        'parallel_experiment_count': 'int',
        'search_space': 'dict(str, object)',
        'start_command': 'str',
        'status': 'str',
        'status_last_updated': 'datetime',
        'status_reason': 'str',
        'suggestion_histories': 'dict(str, object)',
        'sweep_created_by': 'int',
        'sweep_kernel_cluster': 'int',
        'sweep_kernel_cluster_node': 'int',
        'sweep_kernel_image': 'int',
        'sweep_kernel_resource_spec': 'int',
        'sweep_organization': 'int',
        'sweep_project': 'int',
        'updated_dt': 'datetime',
        'volume_mounts': 'dict(str, object)'
    }

    attribute_map = {
        'algorithm': 'algorithm',
        'created_dt': 'created_dt',
        'early_stopping_spec': 'early_stopping_spec',
        'edges': 'edges',
        'env_vars': 'env_vars',
        'final_cost': 'final_cost',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'kernel_cluster_select_policies': 'kernel_cluster_select_policies',
        'max_experiment_count': 'max_experiment_count',
        'max_failed_experiment_count': 'max_failed_experiment_count',
        'message': 'message',
        'name': 'name',
        'objective': 'objective',
        'objective_step_median': 'objective_step_median',
        'parallel_experiment_count': 'parallel_experiment_count',
        'search_space': 'search_space',
        'start_command': 'start_command',
        'status': 'status',
        'status_last_updated': 'status_last_updated',
        'status_reason': 'status_reason',
        'suggestion_histories': 'suggestion_histories',
        'sweep_created_by': 'sweep_created_by',
        'sweep_kernel_cluster': 'sweep_kernel_cluster',
        'sweep_kernel_cluster_node': 'sweep_kernel_cluster_node',
        'sweep_kernel_image': 'sweep_kernel_image',
        'sweep_kernel_resource_spec': 'sweep_kernel_resource_spec',
        'sweep_organization': 'sweep_organization',
        'sweep_project': 'sweep_project',
        'updated_dt': 'updated_dt',
        'volume_mounts': 'volume_mounts'
    }

    def __init__(self, algorithm=None, created_dt=None, early_stopping_spec=None, edges=None, env_vars=None, final_cost=None, id=None, immutable_slug=None, kernel_cluster_select_policies=None, max_experiment_count=None, max_failed_experiment_count=None, message=None, name=None, objective=None, objective_step_median=None, parallel_experiment_count=None, search_space=None, start_command=None, status=None, status_last_updated=None, status_reason=None, suggestion_histories=None, sweep_created_by=None, sweep_kernel_cluster=None, sweep_kernel_cluster_node=None, sweep_kernel_image=None, sweep_kernel_resource_spec=None, sweep_organization=None, sweep_project=None, updated_dt=None, volume_mounts=None, local_vars_configuration=None):  # noqa: E501
        """OrmSweep - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._algorithm = None
        self._created_dt = None
        self._early_stopping_spec = None
        self._edges = None
        self._env_vars = None
        self._final_cost = None
        self._id = None
        self._immutable_slug = None
        self._kernel_cluster_select_policies = None
        self._max_experiment_count = None
        self._max_failed_experiment_count = None
        self._message = None
        self._name = None
        self._objective = None
        self._objective_step_median = None
        self._parallel_experiment_count = None
        self._search_space = None
        self._start_command = None
        self._status = None
        self._status_last_updated = None
        self._status_reason = None
        self._suggestion_histories = None
        self._sweep_created_by = None
        self._sweep_kernel_cluster = None
        self._sweep_kernel_cluster_node = None
        self._sweep_kernel_image = None
        self._sweep_kernel_resource_spec = None
        self._sweep_organization = None
        self._sweep_project = None
        self._updated_dt = None
        self._volume_mounts = None
        self.discriminator = None

        if algorithm is not None:
            self.algorithm = algorithm
        if created_dt is not None:
            self.created_dt = created_dt
        if early_stopping_spec is not None:
            self.early_stopping_spec = early_stopping_spec
        if edges is not None:
            self.edges = edges
        if env_vars is not None:
            self.env_vars = env_vars
        self.final_cost = final_cost
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if kernel_cluster_select_policies is not None:
            self.kernel_cluster_select_policies = kernel_cluster_select_policies
        if max_experiment_count is not None:
            self.max_experiment_count = max_experiment_count
        if max_failed_experiment_count is not None:
            self.max_failed_experiment_count = max_failed_experiment_count
        if message is not None:
            self.message = message
        if name is not None:
            self.name = name
        if objective is not None:
            self.objective = objective
        if objective_step_median is not None:
            self.objective_step_median = objective_step_median
        if parallel_experiment_count is not None:
            self.parallel_experiment_count = parallel_experiment_count
        if search_space is not None:
            self.search_space = search_space
        if start_command is not None:
            self.start_command = start_command
        if status is not None:
            self.status = status
        if status_last_updated is not None:
            self.status_last_updated = status_last_updated
        if status_reason is not None:
            self.status_reason = status_reason
        if suggestion_histories is not None:
            self.suggestion_histories = suggestion_histories
        if sweep_created_by is not None:
            self.sweep_created_by = sweep_created_by
        self.sweep_kernel_cluster = sweep_kernel_cluster
        self.sweep_kernel_cluster_node = sweep_kernel_cluster_node
        if sweep_kernel_image is not None:
            self.sweep_kernel_image = sweep_kernel_image
        self.sweep_kernel_resource_spec = sweep_kernel_resource_spec
        if sweep_organization is not None:
            self.sweep_organization = sweep_organization
        if sweep_project is not None:
            self.sweep_project = sweep_project
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if volume_mounts is not None:
            self.volume_mounts = volume_mounts

    @property
    def algorithm(self):
        """Gets the algorithm of this OrmSweep.  # noqa: E501


        :return: The algorithm of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._algorithm

    @algorithm.setter
    def algorithm(self, algorithm):
        """Sets the algorithm of this OrmSweep.


        :param algorithm: The algorithm of this OrmSweep.  # noqa: E501
        :type algorithm: str
        """

        self._algorithm = algorithm

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmSweep.  # noqa: E501


        :return: The created_dt of this OrmSweep.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmSweep.


        :param created_dt: The created_dt of this OrmSweep.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def early_stopping_spec(self):
        """Gets the early_stopping_spec of this OrmSweep.  # noqa: E501


        :return: The early_stopping_spec of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._early_stopping_spec

    @early_stopping_spec.setter
    def early_stopping_spec(self, early_stopping_spec):
        """Sets the early_stopping_spec of this OrmSweep.


        :param early_stopping_spec: The early_stopping_spec of this OrmSweep.  # noqa: E501
        :type early_stopping_spec: dict(str, object)
        """

        self._early_stopping_spec = early_stopping_spec

    @property
    def edges(self):
        """Gets the edges of this OrmSweep.  # noqa: E501


        :return: The edges of this OrmSweep.  # noqa: E501
        :rtype: OrmSweepEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmSweep.


        :param edges: The edges of this OrmSweep.  # noqa: E501
        :type edges: OrmSweepEdges
        """

        self._edges = edges

    @property
    def env_vars(self):
        """Gets the env_vars of this OrmSweep.  # noqa: E501


        :return: The env_vars of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._env_vars

    @env_vars.setter
    def env_vars(self, env_vars):
        """Sets the env_vars of this OrmSweep.


        :param env_vars: The env_vars of this OrmSweep.  # noqa: E501
        :type env_vars: dict(str, object)
        """

        self._env_vars = env_vars

    @property
    def final_cost(self):
        """Gets the final_cost of this OrmSweep.  # noqa: E501


        :return: The final_cost of this OrmSweep.  # noqa: E501
        :rtype: float
        """
        return self._final_cost

    @final_cost.setter
    def final_cost(self, final_cost):
        """Sets the final_cost of this OrmSweep.


        :param final_cost: The final_cost of this OrmSweep.  # noqa: E501
        :type final_cost: float
        """

        self._final_cost = final_cost

    @property
    def id(self):
        """Gets the id of this OrmSweep.  # noqa: E501


        :return: The id of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmSweep.


        :param id: The id of this OrmSweep.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmSweep.  # noqa: E501


        :return: The immutable_slug of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmSweep.


        :param immutable_slug: The immutable_slug of this OrmSweep.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def kernel_cluster_select_policies(self):
        """Gets the kernel_cluster_select_policies of this OrmSweep.  # noqa: E501


        :return: The kernel_cluster_select_policies of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._kernel_cluster_select_policies

    @kernel_cluster_select_policies.setter
    def kernel_cluster_select_policies(self, kernel_cluster_select_policies):
        """Sets the kernel_cluster_select_policies of this OrmSweep.


        :param kernel_cluster_select_policies: The kernel_cluster_select_policies of this OrmSweep.  # noqa: E501
        :type kernel_cluster_select_policies: dict(str, object)
        """

        self._kernel_cluster_select_policies = kernel_cluster_select_policies

    @property
    def max_experiment_count(self):
        """Gets the max_experiment_count of this OrmSweep.  # noqa: E501


        :return: The max_experiment_count of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._max_experiment_count

    @max_experiment_count.setter
    def max_experiment_count(self, max_experiment_count):
        """Sets the max_experiment_count of this OrmSweep.


        :param max_experiment_count: The max_experiment_count of this OrmSweep.  # noqa: E501
        :type max_experiment_count: int
        """

        self._max_experiment_count = max_experiment_count

    @property
    def max_failed_experiment_count(self):
        """Gets the max_failed_experiment_count of this OrmSweep.  # noqa: E501


        :return: The max_failed_experiment_count of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._max_failed_experiment_count

    @max_failed_experiment_count.setter
    def max_failed_experiment_count(self, max_failed_experiment_count):
        """Sets the max_failed_experiment_count of this OrmSweep.


        :param max_failed_experiment_count: The max_failed_experiment_count of this OrmSweep.  # noqa: E501
        :type max_failed_experiment_count: int
        """

        self._max_failed_experiment_count = max_failed_experiment_count

    @property
    def message(self):
        """Gets the message of this OrmSweep.  # noqa: E501


        :return: The message of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this OrmSweep.


        :param message: The message of this OrmSweep.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def name(self):
        """Gets the name of this OrmSweep.  # noqa: E501


        :return: The name of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmSweep.


        :param name: The name of this OrmSweep.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def objective(self):
        """Gets the objective of this OrmSweep.  # noqa: E501


        :return: The objective of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._objective

    @objective.setter
    def objective(self, objective):
        """Sets the objective of this OrmSweep.


        :param objective: The objective of this OrmSweep.  # noqa: E501
        :type objective: dict(str, object)
        """

        self._objective = objective

    @property
    def objective_step_median(self):
        """Gets the objective_step_median of this OrmSweep.  # noqa: E501


        :return: The objective_step_median of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._objective_step_median

    @objective_step_median.setter
    def objective_step_median(self, objective_step_median):
        """Sets the objective_step_median of this OrmSweep.


        :param objective_step_median: The objective_step_median of this OrmSweep.  # noqa: E501
        :type objective_step_median: dict(str, object)
        """

        self._objective_step_median = objective_step_median

    @property
    def parallel_experiment_count(self):
        """Gets the parallel_experiment_count of this OrmSweep.  # noqa: E501


        :return: The parallel_experiment_count of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._parallel_experiment_count

    @parallel_experiment_count.setter
    def parallel_experiment_count(self, parallel_experiment_count):
        """Sets the parallel_experiment_count of this OrmSweep.


        :param parallel_experiment_count: The parallel_experiment_count of this OrmSweep.  # noqa: E501
        :type parallel_experiment_count: int
        """

        self._parallel_experiment_count = parallel_experiment_count

    @property
    def search_space(self):
        """Gets the search_space of this OrmSweep.  # noqa: E501


        :return: The search_space of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._search_space

    @search_space.setter
    def search_space(self, search_space):
        """Sets the search_space of this OrmSweep.


        :param search_space: The search_space of this OrmSweep.  # noqa: E501
        :type search_space: dict(str, object)
        """

        self._search_space = search_space

    @property
    def start_command(self):
        """Gets the start_command of this OrmSweep.  # noqa: E501


        :return: The start_command of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._start_command

    @start_command.setter
    def start_command(self, start_command):
        """Sets the start_command of this OrmSweep.


        :param start_command: The start_command of this OrmSweep.  # noqa: E501
        :type start_command: str
        """

        self._start_command = start_command

    @property
    def status(self):
        """Gets the status of this OrmSweep.  # noqa: E501


        :return: The status of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this OrmSweep.


        :param status: The status of this OrmSweep.  # noqa: E501
        :type status: str
        """

        self._status = status

    @property
    def status_last_updated(self):
        """Gets the status_last_updated of this OrmSweep.  # noqa: E501


        :return: The status_last_updated of this OrmSweep.  # noqa: E501
        :rtype: datetime
        """
        return self._status_last_updated

    @status_last_updated.setter
    def status_last_updated(self, status_last_updated):
        """Sets the status_last_updated of this OrmSweep.


        :param status_last_updated: The status_last_updated of this OrmSweep.  # noqa: E501
        :type status_last_updated: datetime
        """

        self._status_last_updated = status_last_updated

    @property
    def status_reason(self):
        """Gets the status_reason of this OrmSweep.  # noqa: E501


        :return: The status_reason of this OrmSweep.  # noqa: E501
        :rtype: str
        """
        return self._status_reason

    @status_reason.setter
    def status_reason(self, status_reason):
        """Sets the status_reason of this OrmSweep.


        :param status_reason: The status_reason of this OrmSweep.  # noqa: E501
        :type status_reason: str
        """

        self._status_reason = status_reason

    @property
    def suggestion_histories(self):
        """Gets the suggestion_histories of this OrmSweep.  # noqa: E501


        :return: The suggestion_histories of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._suggestion_histories

    @suggestion_histories.setter
    def suggestion_histories(self, suggestion_histories):
        """Sets the suggestion_histories of this OrmSweep.


        :param suggestion_histories: The suggestion_histories of this OrmSweep.  # noqa: E501
        :type suggestion_histories: dict(str, object)
        """

        self._suggestion_histories = suggestion_histories

    @property
    def sweep_created_by(self):
        """Gets the sweep_created_by of this OrmSweep.  # noqa: E501


        :return: The sweep_created_by of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_created_by

    @sweep_created_by.setter
    def sweep_created_by(self, sweep_created_by):
        """Sets the sweep_created_by of this OrmSweep.


        :param sweep_created_by: The sweep_created_by of this OrmSweep.  # noqa: E501
        :type sweep_created_by: int
        """

        self._sweep_created_by = sweep_created_by

    @property
    def sweep_kernel_cluster(self):
        """Gets the sweep_kernel_cluster of this OrmSweep.  # noqa: E501


        :return: The sweep_kernel_cluster of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_kernel_cluster

    @sweep_kernel_cluster.setter
    def sweep_kernel_cluster(self, sweep_kernel_cluster):
        """Sets the sweep_kernel_cluster of this OrmSweep.


        :param sweep_kernel_cluster: The sweep_kernel_cluster of this OrmSweep.  # noqa: E501
        :type sweep_kernel_cluster: int
        """

        self._sweep_kernel_cluster = sweep_kernel_cluster

    @property
    def sweep_kernel_cluster_node(self):
        """Gets the sweep_kernel_cluster_node of this OrmSweep.  # noqa: E501


        :return: The sweep_kernel_cluster_node of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_kernel_cluster_node

    @sweep_kernel_cluster_node.setter
    def sweep_kernel_cluster_node(self, sweep_kernel_cluster_node):
        """Sets the sweep_kernel_cluster_node of this OrmSweep.


        :param sweep_kernel_cluster_node: The sweep_kernel_cluster_node of this OrmSweep.  # noqa: E501
        :type sweep_kernel_cluster_node: int
        """

        self._sweep_kernel_cluster_node = sweep_kernel_cluster_node

    @property
    def sweep_kernel_image(self):
        """Gets the sweep_kernel_image of this OrmSweep.  # noqa: E501


        :return: The sweep_kernel_image of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_kernel_image

    @sweep_kernel_image.setter
    def sweep_kernel_image(self, sweep_kernel_image):
        """Sets the sweep_kernel_image of this OrmSweep.


        :param sweep_kernel_image: The sweep_kernel_image of this OrmSweep.  # noqa: E501
        :type sweep_kernel_image: int
        """

        self._sweep_kernel_image = sweep_kernel_image

    @property
    def sweep_kernel_resource_spec(self):
        """Gets the sweep_kernel_resource_spec of this OrmSweep.  # noqa: E501


        :return: The sweep_kernel_resource_spec of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_kernel_resource_spec

    @sweep_kernel_resource_spec.setter
    def sweep_kernel_resource_spec(self, sweep_kernel_resource_spec):
        """Sets the sweep_kernel_resource_spec of this OrmSweep.


        :param sweep_kernel_resource_spec: The sweep_kernel_resource_spec of this OrmSweep.  # noqa: E501
        :type sweep_kernel_resource_spec: int
        """

        self._sweep_kernel_resource_spec = sweep_kernel_resource_spec

    @property
    def sweep_organization(self):
        """Gets the sweep_organization of this OrmSweep.  # noqa: E501


        :return: The sweep_organization of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_organization

    @sweep_organization.setter
    def sweep_organization(self, sweep_organization):
        """Sets the sweep_organization of this OrmSweep.


        :param sweep_organization: The sweep_organization of this OrmSweep.  # noqa: E501
        :type sweep_organization: int
        """

        self._sweep_organization = sweep_organization

    @property
    def sweep_project(self):
        """Gets the sweep_project of this OrmSweep.  # noqa: E501


        :return: The sweep_project of this OrmSweep.  # noqa: E501
        :rtype: int
        """
        return self._sweep_project

    @sweep_project.setter
    def sweep_project(self, sweep_project):
        """Sets the sweep_project of this OrmSweep.


        :param sweep_project: The sweep_project of this OrmSweep.  # noqa: E501
        :type sweep_project: int
        """

        self._sweep_project = sweep_project

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmSweep.  # noqa: E501


        :return: The updated_dt of this OrmSweep.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmSweep.


        :param updated_dt: The updated_dt of this OrmSweep.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def volume_mounts(self):
        """Gets the volume_mounts of this OrmSweep.  # noqa: E501


        :return: The volume_mounts of this OrmSweep.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, volume_mounts):
        """Sets the volume_mounts of this OrmSweep.


        :param volume_mounts: The volume_mounts of this OrmSweep.  # noqa: E501
        :type volume_mounts: dict(str, object)
        """

        self._volume_mounts = volume_mounts

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmSweep):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmSweep):
            return True

        return self.to_dict() != other.to_dict()



import datetime
import os
from test.conftest import USE_MOCK

import pytest
from mock import Mock, patch

import savvihub
from savvihub.util.constant import SSH_CONFIG_FORMAT
from savvihub.util.exception import InvalidWorkspaceError


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestWorkspace:
    def test_read_workspace(self):
        # Must create first
        pass

    def test_list_workspaces(self):
        savvihub.list_workspaces()


def test_backup_workspace():
    # Test fail
    config_instance_mock = Mock()
    config_instance_mock.workspace = None
    config_mock = Mock(return_value=config_instance_mock)

    with patch.object(savvihub.workspace, "VesslConfigLoader", config_mock):
        with pytest.raises(InvalidWorkspaceError):
            savvihub.backup_workspace()

    # TODO: Add test success


def test_restore_workspace():
    # Test fail
    config_instance_mock = Mock()
    config_instance_mock.workspace = None
    config_mock = Mock(return_value=config_instance_mock)

    with patch.object(savvihub.workspace, "VesslConfigLoader", config_mock):
        with pytest.raises(InvalidWorkspaceError):
            savvihub.restore_workspace()

    # TODO: Add test success


def test_connect_workspace_ssh():
    workspace_id = 1234
    private_key_path = "/path/to/key"

    # Test fail
    workspace_mock = Mock()
    workspace_mock.status = "terminated"

    read_workspace_ = patch.object(
        savvihub.workspace, "read_workspace", return_value=workspace_mock
    )

    with read_workspace_:
        with pytest.raises(InvalidWorkspaceError):
            savvihub.connect_workspace_ssh(workspace_id, private_key_path)

    # Test success
    port = 1234
    hostname = "tcp.hostname.com"

    workspace_mock = Mock()
    workspace_mock.status = "running"
    workspace_mock.endpoints.ssh.endpoint = f"tcp://{hostname}:{port}"

    read_workspace_ = patch.object(
        savvihub.workspace, "read_workspace", return_value=workspace_mock
    )
    system_ = patch("savvihub.workspace.os.system")

    with read_workspace_, system_:
        savvihub.connect_workspace_ssh(workspace_id, private_key_path)
        savvihub.workspace.os.system.assert_called_with(
            f"ssh -p {port} -i {private_key_path} vessl@{hostname}"
        )


def test_update_vscode_remote_ssh():
    # Setup
    ssh_config_path = "test/ssh_config"
    with open(ssh_config_path, "w+"):
        pass

    private_key_path = "/path/to/key"
    name = "vessl_workspace"
    created_dt = datetime.datetime.now()
    port = 1234
    hostname = "tcp.hostname.com"
    host = f"{name}-{int(created_dt.timestamp())}"

    workspace_mock = Mock()
    workspace_mock.name = name
    workspace_mock.created_dt = created_dt
    workspace_mock.endpoints.ssh.endpoint = f"tcp://{hostname}:{port}"

    list_workspaces_ = patch.object(
        savvihub.workspace, "list_workspaces", return_value=[workspace_mock]
    )
    paramiko_ = patch.object(savvihub.workspace, "paramiko")
    ssh_config_format_ = patch.object(
        savvihub.workspace, "SSH_CONFIG_PATH", ssh_config_path
    )

    with list_workspaces_, paramiko_, ssh_config_format_:
        savvihub.update_vscode_remote_ssh(private_key_path)

    with open(ssh_config_path, "r") as f:
        assert f.read() == SSH_CONFIG_FORMAT.format(
            host=host,
            hostname=hostname,
            port=port,
            private_key_path=private_key_path,
        )

    # Teardown
    try:
        os.remove(ssh_config_path)
    except OSError:
        pass



import warnings

import click

from savvihub.cli._base import VesslGroup
from savvihub.cli._util import print_table, truncate_datetime
from savvihub.cli.organization import organization_name_option
from savvihub.cli.project import project_name_option
from savvihub.model import list_models, read_model
from savvihub.volume import copy_volume_file


@click.command(name="model", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@organization_name_option
@project_name_option
def list():
    models = list_models()
    print_table(
        models,
        ["ID", "Version", "Created", "Status"],
        lambda x: [x.id, x.version, truncate_datetime(x.created_dt), x.status],
    )


@cli.vessl_command()
@click.argument("version", type=click.INT, required=True)
@click.argument("path", type=click.Path(), required=True)
@organization_name_option
@project_name_option
def download(version: int, path: str):
    model = read_model(version=version)
    copy_volume_file(model.artifact_volume_id, "", None, path, True)



import click

from savvihub.cli._base import VesslGroup, vessl_argument
from savvihub.cli._util import (
    generic_prompter,
    print_table,
    prompt_choices,
    truncate_datetime,
)
from savvihub.ssh_key import create_ssh_key, delete_ssh_key, list_ssh_keys


def ssh_key_id_prompter(ctx: click.Context, param: click.Parameter, value: int) -> int:
    ssh_keys = list_ssh_keys()
    return prompt_choices(
        "SSH key", [(f"{x.name} ({x.created_dt})", x.id) for x in ssh_keys]
    )


@click.command(name="ssh-key", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
def list():
    ssh_keys = list_ssh_keys()
    print_table(
        ssh_keys,
        ["ID", "Name", "File", "Created"],
        lambda x: [x.id, x.name, x.filename, truncate_datetime(x.created_dt)],
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=generic_prompter("SSH key name")
)
@vessl_argument(
    "path",
    type=click.Path(exists=True),
    required=True,
    prompter=generic_prompter("Path to public key"),
)
def create(name: str, path: str):
    ssh_key = create_ssh_key(key_name=name, key_path=path)
    print(f"SSH Key '{ssh_key.name}' created.")


@cli.vessl_command()
@vessl_argument("id", type=click.INT, required=True, prompter=ssh_key_id_prompter)
def delete(id: int):
    data = delete_ssh_key(key_id=id)
    print(f"Deleted {id}.")



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseOrganizationCredentialsInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'credentials_id': 'int',
        'credentials_key': 'str',
        'credentials_name': 'str',
        'credentials_type': 'str',
        'organization_id': 'int'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'credentials_id': 'credentials_id',
        'credentials_key': 'credentials_key',
        'credentials_name': 'credentials_name',
        'credentials_type': 'credentials_type',
        'organization_id': 'organization_id'
    }

    def __init__(self, created_dt=None, credentials_id=None, credentials_key=None, credentials_name=None, credentials_type=None, organization_id=None, local_vars_configuration=None):  # noqa: E501
        """ResponseOrganizationCredentialsInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._credentials_id = None
        self._credentials_key = None
        self._credentials_name = None
        self._credentials_type = None
        self._organization_id = None
        self.discriminator = None

        if created_dt is not None:
            self.created_dt = created_dt
        if credentials_id is not None:
            self.credentials_id = credentials_id
        if credentials_key is not None:
            self.credentials_key = credentials_key
        if credentials_name is not None:
            self.credentials_name = credentials_name
        if credentials_type is not None:
            self.credentials_type = credentials_type
        if organization_id is not None:
            self.organization_id = organization_id

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The created_dt of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseOrganizationCredentialsInfo.


        :param created_dt: The created_dt of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def credentials_id(self):
        """Gets the credentials_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The credentials_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: int
        """
        return self._credentials_id

    @credentials_id.setter
    def credentials_id(self, credentials_id):
        """Sets the credentials_id of this ResponseOrganizationCredentialsInfo.


        :param credentials_id: The credentials_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type credentials_id: int
        """

        self._credentials_id = credentials_id

    @property
    def credentials_key(self):
        """Gets the credentials_key of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The credentials_key of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: str
        """
        return self._credentials_key

    @credentials_key.setter
    def credentials_key(self, credentials_key):
        """Sets the credentials_key of this ResponseOrganizationCredentialsInfo.


        :param credentials_key: The credentials_key of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type credentials_key: str
        """

        self._credentials_key = credentials_key

    @property
    def credentials_name(self):
        """Gets the credentials_name of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The credentials_name of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: str
        """
        return self._credentials_name

    @credentials_name.setter
    def credentials_name(self, credentials_name):
        """Sets the credentials_name of this ResponseOrganizationCredentialsInfo.


        :param credentials_name: The credentials_name of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type credentials_name: str
        """

        self._credentials_name = credentials_name

    @property
    def credentials_type(self):
        """Gets the credentials_type of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The credentials_type of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: str
        """
        return self._credentials_type

    @credentials_type.setter
    def credentials_type(self, credentials_type):
        """Sets the credentials_type of this ResponseOrganizationCredentialsInfo.


        :param credentials_type: The credentials_type of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type credentials_type: str
        """

        self._credentials_type = credentials_type

    @property
    def organization_id(self):
        """Gets the organization_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501


        :return: The organization_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ResponseOrganizationCredentialsInfo.


        :param organization_id: The organization_id of this ResponseOrganizationCredentialsInfo.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseOrganizationCredentialsInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseOrganizationCredentialsInfo):
            return True

        return self.to_dict() != other.to_dict()



import click

from savvihub.cli._base import VesslGroup, vessl_argument
from savvihub.cli._util import (
    Endpoint,
    generic_prompter,
    print_data,
    print_table,
    print_volume_files,
    prompt_choices,
)
from savvihub.cli.organization import organization_name_option
from savvihub.dataset import (
    copy_dataset_volume_file,
    create_dataset,
    delete_dataset_volume_file,
    download_dataset_volume_file,
    list_dataset_volume_files,
    list_datasets,
    read_dataset,
    upload_dataset_volume_file,
)


def dataset_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    datasets = list_datasets()
    return prompt_choices("Dataset", [x.name for x in datasets])


@click.command(name="dataset", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=dataset_name_prompter
)
@organization_name_option
def read(name: str):
    dataset = read_dataset(dataset_name=name)
    print_data(
        {
            "ID": dataset.id,
            "Name": dataset.name,
            "Organization": dataset.organization.name,
            "Versioning": dataset.is_version_enabled,
            "Volume ID": dataset.volume_id,
            "Source": dataset.source.type,
        }
    )
    print(
        f"For more info: {Endpoint.dataset.format(dataset.organization.name, dataset.name)}"
    )


@cli.vessl_command()
@organization_name_option
def list():
    datasets = list_datasets()
    print_table(
        datasets,
        ["ID", "Name", "Status", "Files", "Source"],
        lambda x: [x.id, x.name, x.status, x.file_count, x.source.type],
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=generic_prompter("Dataset name")
)
@click.option("-m", "--description", type=click.STRING)
@click.option(
    "-e",
    "--external-path",
    type=click.STRING,
    help="AWS S3 or Google Cloud Storage bucket URL (starts with `s3://` or `gs://`).",
)
@click.option("--aws-role-arn", type=click.STRING, help="Required for S3 datasets.")
@click.option("--enable-versioning", is_flag=True)
@click.option(
    "--version-path",
    type=click.STRING,
    help="Required to version AWS S3 or Google Cloud Storage dataset.",
)
@organization_name_option
def create(
    name: str,
    description: str,
    external_path: str,
    aws_role_arn: str,
    enable_versioning: bool,
    version_path: str,
):
    dataset = create_dataset(
        dataset_name=name,
        description=description,
        is_version_enabled=enable_versioning,
        external_path=external_path,
        aws_role_arn=aws_role_arn,
        version_path=version_path,
    )
    print(
        f"Created '{dataset.name}'."
        f"For more info: {Endpoint.dataset.format(dataset.organization.name, dataset.name)}"
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=dataset_name_prompter
)
@click.option("-p", "--path", type=click.Path(), default="", help="Defaults to root.")
@click.option("-r", "--recursive", is_flag=True)
@organization_name_option
def list_files(
    name: str,
    path: str,
    recursive: bool,
):
    files = list_dataset_volume_files(
        dataset_name=name,
        path=path,
        need_download_url=False,
        recursive=recursive,
    )
    print_volume_files(files)


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=dataset_name_prompter,
)
@vessl_argument(
    "source",
    type=click.Path(exists=True),
    required=True,
    prompter=generic_prompter("Source path"),
)
@vessl_argument(
    "dest",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Destination path"),
)
@organization_name_option
def upload(name: str, source: str, dest: str):
    upload_dataset_volume_file(
        dataset_name=name,
        source_path=source,
        dest_path=dest,
    )
    print(f"Uploaded {source} to {dest}.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=dataset_name_prompter
)
@vessl_argument(
    "source", type=click.Path(), required=True, prompter=generic_prompter("Source path")
)
@vessl_argument(
    "dest",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Destination path"),
)
@organization_name_option
def download(name: str, source: str, dest: str):
    download_dataset_volume_file(
        dataset_name=name,
        source_path=source,
        dest_path=dest,
    )
    print(f"Downloaded {source} to {dest}.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=dataset_name_prompter
)
@vessl_argument(
    "source", type=click.Path(), required=True, prompter=generic_prompter("Source path")
)
@vessl_argument(
    "dest",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Destination path"),
)
@click.option("-r", "--recursive", is_flag=True, help="Required if directory.")
@organization_name_option
def copy(name: str, source: str, dest: str, recursive: bool):
    copy_dataset_volume_file(
        dataset_name=name,
        source_path=source,
        dest_path=dest,
        recursive=recursive,
    )
    print(f"Copied {source} to {dest}.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=dataset_name_prompter
)
@vessl_argument(
    "path", type=click.Path(), required=True, prompter=generic_prompter("File path")
)
@click.option("-r", "--recursive", is_flag=True, help="Required if directory.")
@organization_name_option
def delete_file(name: str, path: str, recursive: bool):
    files = delete_dataset_volume_file(
        dataset_name=name,
        path=path,
        recursive=recursive,
    )
    print(f"Deleted {path} ({len([x for x in files if not x.is_dir])} file(s)).")



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmExperiment(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'edges': 'OrmExperimentEdges',
        'experiment_plot_volume': 'int',
        'experiment_workload': 'int',
        'id': 'int',
        'metrics_summary': 'dict(str, object)',
        'number': 'int',
        'termination_protection': 'bool'
    }

    attribute_map = {
        'edges': 'edges',
        'experiment_plot_volume': 'experiment_plot_volume',
        'experiment_workload': 'experiment_workload',
        'id': 'id',
        'metrics_summary': 'metrics_summary',
        'number': 'number',
        'termination_protection': 'termination_protection'
    }

    def __init__(self, edges=None, experiment_plot_volume=None, experiment_workload=None, id=None, metrics_summary=None, number=None, termination_protection=None, local_vars_configuration=None):  # noqa: E501
        """OrmExperiment - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._edges = None
        self._experiment_plot_volume = None
        self._experiment_workload = None
        self._id = None
        self._metrics_summary = None
        self._number = None
        self._termination_protection = None
        self.discriminator = None

        if edges is not None:
            self.edges = edges
        if experiment_plot_volume is not None:
            self.experiment_plot_volume = experiment_plot_volume
        if experiment_workload is not None:
            self.experiment_workload = experiment_workload
        if id is not None:
            self.id = id
        if metrics_summary is not None:
            self.metrics_summary = metrics_summary
        if number is not None:
            self.number = number
        if termination_protection is not None:
            self.termination_protection = termination_protection

    @property
    def edges(self):
        """Gets the edges of this OrmExperiment.  # noqa: E501


        :return: The edges of this OrmExperiment.  # noqa: E501
        :rtype: OrmExperimentEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmExperiment.


        :param edges: The edges of this OrmExperiment.  # noqa: E501
        :type edges: OrmExperimentEdges
        """

        self._edges = edges

    @property
    def experiment_plot_volume(self):
        """Gets the experiment_plot_volume of this OrmExperiment.  # noqa: E501


        :return: The experiment_plot_volume of this OrmExperiment.  # noqa: E501
        :rtype: int
        """
        return self._experiment_plot_volume

    @experiment_plot_volume.setter
    def experiment_plot_volume(self, experiment_plot_volume):
        """Sets the experiment_plot_volume of this OrmExperiment.


        :param experiment_plot_volume: The experiment_plot_volume of this OrmExperiment.  # noqa: E501
        :type experiment_plot_volume: int
        """

        self._experiment_plot_volume = experiment_plot_volume

    @property
    def experiment_workload(self):
        """Gets the experiment_workload of this OrmExperiment.  # noqa: E501


        :return: The experiment_workload of this OrmExperiment.  # noqa: E501
        :rtype: int
        """
        return self._experiment_workload

    @experiment_workload.setter
    def experiment_workload(self, experiment_workload):
        """Sets the experiment_workload of this OrmExperiment.


        :param experiment_workload: The experiment_workload of this OrmExperiment.  # noqa: E501
        :type experiment_workload: int
        """

        self._experiment_workload = experiment_workload

    @property
    def id(self):
        """Gets the id of this OrmExperiment.  # noqa: E501


        :return: The id of this OrmExperiment.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmExperiment.


        :param id: The id of this OrmExperiment.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def metrics_summary(self):
        """Gets the metrics_summary of this OrmExperiment.  # noqa: E501


        :return: The metrics_summary of this OrmExperiment.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._metrics_summary

    @metrics_summary.setter
    def metrics_summary(self, metrics_summary):
        """Sets the metrics_summary of this OrmExperiment.


        :param metrics_summary: The metrics_summary of this OrmExperiment.  # noqa: E501
        :type metrics_summary: dict(str, object)
        """

        self._metrics_summary = metrics_summary

    @property
    def number(self):
        """Gets the number of this OrmExperiment.  # noqa: E501


        :return: The number of this OrmExperiment.  # noqa: E501
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """Sets the number of this OrmExperiment.


        :param number: The number of this OrmExperiment.  # noqa: E501
        :type number: int
        """

        self._number = number

    @property
    def termination_protection(self):
        """Gets the termination_protection of this OrmExperiment.  # noqa: E501


        :return: The termination_protection of this OrmExperiment.  # noqa: E501
        :rtype: bool
        """
        return self._termination_protection

    @termination_protection.setter
    def termination_protection(self, termination_protection):
        """Sets the termination_protection of this OrmExperiment.


        :param termination_protection: The termination_protection of this OrmExperiment.  # noqa: E501
        :type termination_protection: bool
        """

        self._termination_protection = termination_protection

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmExperiment):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmExperiment):
            return True

        return self.to_dict() != other.to_dict()



import click
import sentry_sdk
from click.decorators import pass_context

import savvihub
from savvihub.cli._base import VesslGroup
from savvihub.cli.dataset import cli as dataset_cli
from savvihub.cli.experiment import cli as experiment_cli
from savvihub.cli.kernel_cluster import cli as kernel_cluster_cli
from savvihub.cli.kernel_image import cli as kernel_image_cli
from savvihub.cli.kernel_resource_spec import cli as kernel_resource_spec_cli
from savvihub.cli.model import cli as model_cli
from savvihub.cli.organization import cli as organization_cli
from savvihub.cli.project import cli as project_cli
from savvihub.cli.ssh_key import cli as ssh_key_cli
from savvihub.cli.sweep import cli as sweep_cli
from savvihub.cli.volume import cli as volume_cli
from savvihub.cli.workspace import cli as workspace_cli
from savvihub.util.config import VesslConfigLoader
from savvihub.util.exception import InvalidTokenError, SavvihubApiException

# Configure Sentry
sentry_sdk.init(
    "https://e46fcd750b3a443fbd5b9dbc970e4ecf@o386227.ingest.sentry.io/5911639",
    traces_sample_rate=1.0,
)


@click.command(cls=VesslGroup)
@click.version_option()
@pass_context
def cli(ctx: click.Context):
    ctx.ensure_object(dict)


access_token_option = click.option("-t", "--access-token", type=click.STRING)
organization_option = click.option("-o", "--organization", type=click.STRING)
project_option = click.option("-p", "--project", type=click.STRING)
credentials_file_option = click.option("-f", "--credentials-file", type=click.STRING)
force_update_option = click.option("--renew-token", is_flag=True)


def _login(access_token: str, credentials_file: str, renew_token: bool):
    try:
        savvihub.update_access_token(
            access_token=access_token,
            credentials_file=credentials_file,
            force_update=renew_token,
        )
    except InvalidTokenError:
        savvihub.update_access_token(force_update=True)
    print(f"Welcome, {savvihub.vessl_api.user.display_name}!")


@cli.command()
@access_token_option
@credentials_file_option
@force_update_option
def login(access_token: str, credentials_file: str, renew_token: bool):
    _login(access_token, credentials_file, renew_token)


@cli.command()
@access_token_option
@organization_option
@project_option
@credentials_file_option
@force_update_option
def init(access_token, organization, project, credentials_file, renew_token):
    _login(access_token, credentials_file, renew_token)

    savvihub.update_organization(
        organization_name=organization, credentials_file=credentials_file
    )

    if project is not None:
        savvihub.update_project(project_name=project, credentials_file=credentials_file)


@cli.command()
def status():
    config = VesslConfigLoader()

    username = ""
    email = ""
    organization = config.default_organization or ""
    project = config.default_project or ""

    if config.access_token:
        savvihub.vessl_api.api_client.set_default_header(
            "Authorization", f"Token {config.access_token}"
        )

        try:
            user = savvihub.vessl_api.get_my_user_info_api()
            username = user.username
            email = user.email
        except SavvihubApiException as e:
            pass

    print(
        f"Username: {username}\n"
        f"Email: {email}\n"
        f"Organization: {organization}\n"
        f"Project: {project}"
    )


cli.add_command(dataset_cli)
cli.add_command(experiment_cli)
cli.add_command(kernel_cluster_cli)
cli.add_command(kernel_image_cli)
cli.add_command(kernel_resource_spec_cli)
cli.add_command(model_cli)
cli.add_command(organization_cli)
cli.add_command(project_cli)
cli.add_command(ssh_key_cli)
cli.add_command(sweep_cli)
cli.add_command(volume_cli)
cli.add_command(workspace_cli)


if __name__ == "__main__":
    cli()



import os
from test.conftest import USE_MOCK
from test.util import random_string

import pytest
from Crypto.PublicKey import RSA

import savvihub


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestSshKey:
    @pytest.mark.order(index=1)
    def test_create_ssh_key(self):
        key_path = "test/fixture/pubkey"
        key = RSA.generate(1024).publickey()
        with open(key_path, "wb") as f:
            f.write(key.exportKey("OpenSSH"))

        key_name = random_string()
        pytest.key = savvihub.create_ssh_key(key_path, key_name)
        os.remove(key_path)

    def test_list_ssh_keys(self):
        savvihub.list_ssh_keys()

    def test_delete_ssh_key(self):
        savvihub.delete_ssh_key(pytest.key.id)



import click

from savvihub.cli._base import VesslGroup, vessl_argument, vessl_option
from savvihub.cli._util import generic_prompter, print_volume_files
from savvihub.volume import copy_volume_file, delete_volume_file, list_volume_files


@click.command(name="volume", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=generic_prompter("Volume ID", click.INT),
)
@click.option("-p", "--path", type=click.Path(), default="", help="Defaults to root.")
@click.option("-r", "--recursive", is_flag=True)
def list(id: int, path: str, recursive: bool):
    files = list_volume_files(
        volume_id=id,
        path=path,
        need_download_url=False,
        recursive=recursive,
    )
    print_volume_files(files)


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=generic_prompter("Volume ID", click.INT),
)
@click.option("-r", "--recursive", is_flag=True, help="Required if directory.")
def delete(id: int, recursive: bool):
    files = delete_volume_file(volume_id=id, recursive=recursive)
    print(f"Deleted {id} ({len([x for x in files if not x.is_dir])} file(s)).")


@cli.vessl_command()
@vessl_option(
    "--source-id",
    type=click.INT,
    prompter=generic_prompter("Source volume ID", click.INT),
    help="If not specified, source is assumed to be local.",
)
@vessl_option(
    "--source-path",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Source path"),
)
@vessl_option(
    "--dest-id",
    type=click.INT,
    prompter=generic_prompter("Destination volume ID", click.INT),
    help="If not specified, destination is assumed to be local.",
)
@vessl_option(
    "--dest-path",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Destination path"),
)
@click.option("-r", "--recursive", is_flag=True, help="Required if directory.")
def copy(
    source_id: int, source_path: str, dest_id: int, dest_path: str, recursive: bool
):
    copy_volume_file(
        source_volume_id=source_id,
        source_path=source_path,
        dest_volume_id=dest_id,
        dest_path=dest_path,
        recursive=recursive,
    )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class StorageFileActionURLInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'body': 'list[str]',
        'federation_token': 'StorageFederationToken',
        'headers': 'list[str]',
        'method': 'str',
        'url': 'str'
    }

    attribute_map = {
        'body': 'body',
        'federation_token': 'federation_token',
        'headers': 'headers',
        'method': 'method',
        'url': 'url'
    }

    def __init__(self, body=None, federation_token=None, headers=None, method=None, url=None, local_vars_configuration=None):  # noqa: E501
        """StorageFileActionURLInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._body = None
        self._federation_token = None
        self._headers = None
        self._method = None
        self._url = None
        self.discriminator = None

        if body is not None:
            self.body = body
        if federation_token is not None:
            self.federation_token = federation_token
        if headers is not None:
            self.headers = headers
        if method is not None:
            self.method = method
        if url is not None:
            self.url = url

    @property
    def body(self):
        """Gets the body of this StorageFileActionURLInfo.  # noqa: E501


        :return: The body of this StorageFileActionURLInfo.  # noqa: E501
        :rtype: list[str]
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this StorageFileActionURLInfo.


        :param body: The body of this StorageFileActionURLInfo.  # noqa: E501
        :type body: list[str]
        """

        self._body = body

    @property
    def federation_token(self):
        """Gets the federation_token of this StorageFileActionURLInfo.  # noqa: E501


        :return: The federation_token of this StorageFileActionURLInfo.  # noqa: E501
        :rtype: StorageFederationToken
        """
        return self._federation_token

    @federation_token.setter
    def federation_token(self, federation_token):
        """Sets the federation_token of this StorageFileActionURLInfo.


        :param federation_token: The federation_token of this StorageFileActionURLInfo.  # noqa: E501
        :type federation_token: StorageFederationToken
        """

        self._federation_token = federation_token

    @property
    def headers(self):
        """Gets the headers of this StorageFileActionURLInfo.  # noqa: E501


        :return: The headers of this StorageFileActionURLInfo.  # noqa: E501
        :rtype: list[str]
        """
        return self._headers

    @headers.setter
    def headers(self, headers):
        """Sets the headers of this StorageFileActionURLInfo.


        :param headers: The headers of this StorageFileActionURLInfo.  # noqa: E501
        :type headers: list[str]
        """

        self._headers = headers

    @property
    def method(self):
        """Gets the method of this StorageFileActionURLInfo.  # noqa: E501


        :return: The method of this StorageFileActionURLInfo.  # noqa: E501
        :rtype: str
        """
        return self._method

    @method.setter
    def method(self, method):
        """Sets the method of this StorageFileActionURLInfo.


        :param method: The method of this StorageFileActionURLInfo.  # noqa: E501
        :type method: str
        """

        self._method = method

    @property
    def url(self):
        """Gets the url of this StorageFileActionURLInfo.  # noqa: E501


        :return: The url of this StorageFileActionURLInfo.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this StorageFileActionURLInfo.


        :param url: The url of this StorageFileActionURLInfo.  # noqa: E501
        :type url: str
        """

        self._url = url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StorageFileActionURLInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StorageFileActionURLInfo):
            return True

        return self.to_dict() != other.to_dict()



import concurrent.futures
import os

from requests_futures.sessions import FuturesSession

from savvihub.util.constant import PARALLEL_WORKERS
from savvihub.util.file_object import DownloadableFileObject


class Downloader:
    @classmethod
    def download(cls, local_path, file, progressable=None):
        dirname = os.path.dirname(local_path)
        basename = os.path.basename(local_path)
        d = DownloadableFileObject(file.download_url.url, dirname, basename)
        return d.download(progressable=progressable)

    @classmethod
    def bulk_download(cls, local_base_path, remote_files, progressable=None):
        if len(remote_files) <= 0:
            return []

        session = FuturesSession(max_workers=PARALLEL_WORKERS)
        futures = []
        for remote_file in remote_files:
            if remote_file.is_dir:
                continue

            d = DownloadableFileObject(
                remote_file.download_url.url, local_base_path, remote_file.path
            )
            futures.append(d.download(session, progressable=progressable))

        concurrent.futures.wait(futures)
        return [future.result() for future in futures]



import shutil
from test.conftest import USE_MOCK
from test.util import random_string

import pytest

import savvihub
from savvihub.util.exception import SavvihubApiException


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestDataset:
    dataset_name = random_string()

    @classmethod
    def teardown_class(cls):
        shutil.rmtree("test/fixture/tmp/", ignore_errors=True)

    @pytest.mark.order(index=1)
    def test_create_dataset(self):
        savvihub.create_dataset(
            dataset_name=self.dataset_name,
            description=random_string(),
        )
        # TODO: test s3 and gs datasets

    @pytest.mark.order(index=2)
    def test_upload_dataset_volume_file(self):
        savvihub.upload_dataset_volume_file(
            self.dataset_name, "test/fixture/file1.txt", "file1.txt"
        )
        savvihub.upload_dataset_volume_file(
            self.dataset_name, "test/fixture/folder1", "folder1", recursive=True
        )

    def test_read_dataset(self):
        savvihub.read_dataset(self.dataset_name)

    def test_list_datasets(self):
        savvihub.list_datasets()

    def test_list_dataset_volume_files(self):
        savvihub.list_dataset_volume_files(self.dataset_name)

    def test_download_dataset_volume_file(self):
        savvihub.download_dataset_volume_file(
            self.dataset_name, "file1.txt", "test/fixture/tmp/file2.txt"
        )
        savvihub.download_dataset_volume_file(
            self.dataset_name, "folder1", "test/fixture/tmp/folder1", recursive=True
        )

    def test_copy_dataset_volume_file(self):
        savvihub.copy_dataset_volume_file(
            self.dataset_name, "file1.txt", "file1_copy.txt"
        )
        savvihub.copy_dataset_volume_file(
            self.dataset_name, "folder1", "folder1_copy", recursive=True
        )
        with pytest.raises(SavvihubApiException):
            savvihub.copy_dataset_volume_file(
                self.dataset_name, "folder1", "folder1_copy"
            )

    @pytest.mark.order(index=-1)
    def test_delete_dataset_volume_file(self):
        savvihub.delete_dataset_volume_file(self.dataset_name, "file1.txt")
        savvihub.delete_dataset_volume_file(
            self.dataset_name, "folder1", recursive=True
        )
        with pytest.raises(SavvihubApiException):
            savvihub.delete_dataset_volume_file(self.dataset_name, "folder1")
